/*
 * RLC.c
 * 
 * Copyright 2023 Reinhard <reinhard@reinhard-TUXEDO>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 * 
 * 
 */

/////////////// HOW TO BUILD ///////////////////
//   flexspin -2 -Wall -DPROPELLER2 RLC.c _complex.c 
///////////////////////////////////////////////////
#include <stdio.h>
#include "_complex.h"
#ifdef PROPELLER2
	#include <propeller2.h>
	// SMARTPINS
	#define CH1  0 // Oszi X
	#define CH2  2 // Oszi Y
	int dac0=0x4000,dac1=0x4000;

    void pinconfig(int pin, int mode, int x, int y)
    {
    _dirl(pin);
    _wrpin(pin,mode);
    _wxpin(pin,x);
    _wypin(pin,y);
    _pinl(pin);
    }
#endif

// R  - L - C  circuit
double F_RLC (_complex_t s)
{
	double R=1.0,L=0.001,C=0.0001;
	_complex_t XL,XC,X,Y,ONE;
	ONE =  CSET(1,0);
	XL  =  CMULD(s,L);
	XC =   CDIV(ONE,CMULD(s,C));
	X  =   CADD(XL,XC);
    Y  =   CDIV(X,CADDD(X,R));
    return CABS(Y);
}

///////////////////////////////////////
//   frequency response  of RLC Circuit   //
///////////////////////////////////////
void demo2 ()
{
	_complex_t  I =    CSET(0.0,1.0);
	_complex_t  s   =  CSET(1.0,1.0);
	int n=0;
	double res[150];
	for (double f = 0.01; f < 1500.0; f+= 10.0)
	{
		double w = 2 * M_PI * f;
		s = CMULD(I,w);  // angular frequency
		res[n] = F_RLC(s);
#ifndef PROPELLER2
		printf("%f %f \n",f,res[n]);
#endif
		n++;
	}

#ifdef PROPELLER2
	// Oszi CH1 <--> PIN 0
	// 500 mV/Div
	// Coupling AC
	// Trigger Level -220 mV
	// Horizontal 80µs/DIV
	// Oszi CH2 <-> PIN2
	// ramp represent the frequency window from DC to 1500 Hz
	while(1)
	{
		for (int k=0;k<150;k++)
		{
			dac0 = (int) (  res[k]  * 30000 ) + 32768;
			dac1 = 256 * k;
			_wypin(CH1,dac0);
			_wypin(CH2,dac1);
		}
	}
#endif
}



int main()
{
#ifdef PROPELLER2
    pinconfig(CH1, 0b101000000000001000100, 1, dac0);  // DAC16bit dither
    _waitms(500);
    pinconfig(CH2, 0b101000000000001000100, 1, dac1);  // DAC16bit dither
    _waitms(500);
#endif	
	demo2();
	return 0;
}

