/*
 * _complex.c
 * 
 * Copyright 2023 Reinhard <reinhard@reinhard-TUXEDO>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 * 
 * 
 */

#include "_complex.h"
_complex_t _complex_add(_complex_t a, _complex_t b)
{
    _complex_t r;
	r.re = a.re + b.re;
	r.im = a.im + b.im;
	return r;
}

_complex_t _complex_sub(_complex_t a, _complex_t b)
{
    _complex_t r;
	r.re = a.re - b.re;
	r.im = a.im - b.im;
	return r;
}

_complex_t _complex_addd(_complex_t a, double b)
{
	_complex_t r;
	r.re = a.re + b;
	r.im = a.im ;
	return r;
}

_complex_t _complex_conj(_complex_t q)
{
	_complex_t r;
	r.re = q.re;
	r.im = -q.im;
	return r;
}

_complex_t _complex_mul(_complex_t a, _complex_t b)
{
	_complex_t r;
	r.re =  a.re  * b.re   - a.im * b.im;
	r.im =  a.re  * b.im   + b.re * a.im;
	return r;
}

_complex_t _complex_muld(_complex_t a, double b)
{
    _complex_t r;
	r.re =  a.re  * b;
	r.im =  a.im  * b;
	return r;
}

double _complex_norm(_complex_t q)
{
	return sqrt(q.re * q.re + q.im * q.im);
}

double _complex_arg(_complex_t q)
{
	return atan2(q.im,q.re);
}

_complex_t _complex_div (_complex_t a, _complex_t b)
{
	_complex_t r;
    _complex_t temp1;
	_complex_t temp2;
	double nenner = (b.re * b.re + b.im * b.im);
	temp1 = _complex_conj(b);
	temp2 =_complex_mul(temp1,a);
	r.re = temp2.re / nenner;
	r.im = temp2.im / nenner;
    return r;
}

_complex_t _complex_exp(_complex_t q)
{
	double A;
    _complex_t r;
	A = exp(q.re) ;
	r.re =  A * cos(q.im);
	r.im =  A * sin(q.im);
	return r;
}

_complex_t _complex_set(double re , double im)
{
    _complex_t r;
    r.re = re;
    r.im = im;
    return r;
}

void _complex_print(_complex_t q, const char * s)
{
	printf("%s = %.4f %+.4fi\n", s,q.re, q.im);
}

void _complex_printf(_complex_t q)
{
	printf("%.4f %4f\n", q.re, q.im);
}

int _complex_equal(_complex_t a, _complex_t b)
{
	//_complex_printf(a),_complex_printf(b);
	//if (a.re != b.re) return -1;
	//if (a.im != b.im) return -1;
	if(abs(a.re - b.re) > 0.0001) return -1;
	if(abs(a.im - b.im) > 0.0001) return -1;
	return 0;
}

void _complex2polar(double  * _abs, double *  _arg, _complex_t q)
{
	*_abs = sqrt(q.re * q.re + q.im * q.im);
	*_arg = atan2(q.im,q.re);
}

_complex_t _polar2complex(double  _abs, double  _arg)
{
	_complex_t q;
	q.re = _abs * cos(_arg);
	q.im = _abs * sin(_arg);
	return q;
}


