/*
 * _complex.h  2023/06/14
 * 
 * Copyright 2023 Reinhard <reinhard@reinhard-TUXEDO>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 * 
 * 
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <math.h>

typedef struct _complex
{
	double re;
	double im;
} _complex_t;

_complex_t _complex_set(double re , double im);
_complex_t _complex_add(_complex_t a, _complex_t b);
_complex_t _complex_sub(_complex_t a, _complex_t b);
_complex_t _complex_addd(_complex_t a, double b);
_complex_t _complex_conj(_complex_t q);
_complex_t _complex_mul(_complex_t a, _complex_t b);
_complex_t _complex_muld(_complex_t a, double b);
double _complex_norm(_complex_t q);
double _complex_arg(_complex_t q);
_complex_t _complex_div (_complex_t a, _complex_t b);
_complex_t _complex_exp(_complex_t q);
void _complex_print(_complex_t q, const char * s);
void _complex_printf(_complex_t q);
int _complex_equal(_complex_t a, _complex_t b);
void _complex2polar(double  * _abs, double *  _arg, _complex_t q);
_complex_t _polar2complex(double  _abs, double  _arg);


#ifndef M_PI
#define M_PI 3.14159265358979323846
#endif

#define CONJ(A)     _complex_conj(A)
#define CEXP(A)     _complex_exp(A)
#define CMULD(A,b)  _complex_muld(A,b)
#define CMUL(A,B)   _complex_mul(A,B)
#define CADD(A,B)   _complex_add(A,B)
#define CADDD(A,b)  _complex_addd(A,b)
#define CABS(A)     _complex_norm(A)
#define CDIV(A,B)   _complex_div(A,B)
#define CSUB(A,B)   _complex_sub(A,B)
#define CSET(a,b)   _complex_set(a,b)




